package testing

const (
	ListRuleTypesResponse = `
{
    "rule_types": [
        {
            "type": "bandwidth_limit"
        },
        {
            "type": "dscp_marking"
        },
        {
            "type": "minimum_bandwidth"
        }
    ]
}
`
)
