// Package instances provides information and interaction with the instance API
// resource in the OpenStack Database service.
//
// A database instance is an isolated database environment with compute and
// storage resources in a single tenant environment on a shared physical host
// machine.
package instances
