// Package capsules contains functionality for working with Zun capsule
// resources. A capsule is a container group, as the co-located and
// co-scheduled unit, is the same like pod in Kubernetes.
package capsules
