package testing

// ImportGetResult represents raw server response on a Get request.
const ImportGetResult = `
{
    "import-methods": {
        "description": "Import methods available.",
        "type": "array",
        "value": [
            "glance-direct",
            "web-download"
        ]
    }
}
`
