// Package flavors provides information and interaction with the flavor API
// resource in the OpenStack Database service.
//
// A flavor is an available hardware configuration for a database instance.
// Each flavor has a unique combination of disk space, memory capacity and
// priority for CPU time.
package flavors
